@@ -17,34 +17,36 @@ import (
 // We define these as pointers types so we can use reflection to check that
 // exactly one is set.
 type StageConfig struct {
-	CRIConfig             *CRIConfig             `alloy:"cri,block,optional"`
-	DecolorizeConfig      *DecolorizeConfig      `alloy:"decolorize,block,optional"`
-	DockerConfig          *DockerConfig          `alloy:"docker,block,optional"`
-	DropConfig            *DropConfig            `alloy:"drop,block,optional"`
-	EventLogMessageConfig *EventLogMessageConfig `alloy:"eventlogmessage,block,optional"`
-	GeoIPConfig           *GeoIPConfig           `alloy:"geoip,block,optional"`
-	JSONConfig            *JSONConfig            `alloy:"json,block,optional"`
-	LabelAllowConfig      *LabelAllowConfig      `alloy:"label_keep,block,optional"`
-	LabelDropConfig       *LabelDropConfig       `alloy:"label_drop,block,optional"`
-	LabelsConfig          *LabelsConfig          `alloy:"labels,block,optional"`
-	LimitConfig           *LimitConfig           `alloy:"limit,block,optional"`
-	LogfmtConfig          *LogfmtConfig          `alloy:"logfmt,block,optional"`
-	LuhnFilterConfig      *LuhnFilterConfig      `alloy:"luhn,block,optional"`
-	MatchConfig           *MatchConfig           `alloy:"match,block,optional"`
-	MetricsConfig         *MetricsConfig         `alloy:"metrics,block,optional"`
-	MultilineConfig       *MultilineConfig       `alloy:"multiline,block,optional"`
-	OutputConfig          *OutputConfig          `alloy:"output,block,optional"`
-	PackConfig            *PackConfig            `alloy:"pack,block,optional"`
-	PatternConfig         *PatternConfig         `alloy:"pattern,block,optional"`
-	RegexConfig           *RegexConfig           `alloy:"regex,block,optional"`
-	ReplaceConfig         *ReplaceConfig         `alloy:"replace,block,optional"`
-	StaticLabelsConfig    *StaticLabelsConfig    `alloy:"static_labels,block,optional"`
-	StructuredMetadata    *LabelsConfig          `alloy:"structured_metadata,block,optional"`
-	SamplingConfig        *SamplingConfig        `alloy:"sampling,block,optional"`
-	TemplateConfig        *TemplateConfig        `alloy:"template,block,optional"`
-	TenantConfig          *TenantConfig          `alloy:"tenant,block,optional"`
-	TimestampConfig       *TimestampConfig       `alloy:"timestamp,block,optional"`
-	WindowsEventConfig    *WindowsEventConfig    `alloy:"windowsevent,block,optional"`
+	CRIConfig                    *CRIConfig                    `alloy:"cri,block,optional"`
+	DecolorizeConfig             *DecolorizeConfig             `alloy:"decolorize,block,optional"`
+	DockerConfig                 *DockerConfig                 `alloy:"docker,block,optional"`
+	DropConfig                   *DropConfig                   `alloy:"drop,block,optional"`
+	EventLogMessageConfig        *EventLogMessageConfig        `alloy:"eventlogmessage,block,optional"`
+	GeoIPConfig                  *GeoIPConfig                  `alloy:"geoip,block,optional"`
+	JSONConfig                   *JSONConfig                   `alloy:"json,block,optional"`
+	LabelAllowConfig             *LabelAllowConfig             `alloy:"label_keep,block,optional"`
+	LabelDropConfig              *LabelDropConfig              `alloy:"label_drop,block,optional"`
+	LabelsConfig                 *LabelsConfig                 `alloy:"labels,block,optional"`
+	LimitConfig                  *LimitConfig                  `alloy:"limit,block,optional"`
+	LogfmtConfig                 *LogfmtConfig                 `alloy:"logfmt,block,optional"`
+	LuhnFilterConfig             *LuhnFilterConfig             `alloy:"luhn,block,optional"`
+	MatchConfig                  *MatchConfig                  `alloy:"match,block,optional"`
+	MetricsConfig                *MetricsConfig                `alloy:"metrics,block,optional"`
+	MultilineConfig              *MultilineConfig              `alloy:"multiline,block,optional"`
+	OutputConfig                 *OutputConfig                 `alloy:"output,block,optional"`
+	PackConfig                   *PackConfig                   `alloy:"pack,block,optional"`
+	PatternConfig                *PatternConfig                `alloy:"pattern,block,optional"`
+	RegexConfig                  *RegexConfig                  `alloy:"regex,block,optional"`
+	ReplaceConfig                *ReplaceConfig                `alloy:"replace,block,optional"`
+	StaticLabelsConfig           *StaticLabelsConfig           `alloy:"static_labels,block,optional"`
+	StructuredMetadata           *StructuredMetadataConfig     `alloy:"structured_metadata,block,optional"`
+	StructuredMetadataDropConfig *StructuredMetadataDropConfig `alloy:"structured_metadata_drop,block,optional"`
+	SamplingConfig               *SamplingConfig               `alloy:"sampling,block,optional"`
+	TemplateConfig               *TemplateConfig               `alloy:"template,block,optional"`
+	TenantConfig                 *TenantConfig                 `alloy:"tenant,block,optional"`
+	TruncateConfig               *TruncateConfig               `alloy:"truncate,block,optional"`
+	TimestampConfig              *TimestampConfig              `alloy:"timestamp,block,optional"`
+	WindowsEventConfig           *WindowsEventConfig           `alloy:"windowsevent,block,optional"`
 }
 
 var rateLimiter *rate.Limiter
