@@ -6,6 +6,7 @@ package file
 import (
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"os"
@@ -13,19 +14,17 @@ import (
 	"time"
 
 	"github.com/go-kit/log"
-	"github.com/grafana/loki/v3/pkg/logproto"
-	"github.com/grafana/loki/v3/pkg/util"
-	"github.com/grafana/tail"
-	"github.com/grafana/tail/watch"
+	"github.com/grafana/loki/pkg/push"
 	"github.com/prometheus/common/model"
 	"go.uber.org/atomic"
 	"golang.org/x/text/encoding"
 	"golang.org/x/text/encoding/ianaindex"
-	"golang.org/x/text/transform"
 
 	"github.com/grafana/alloy/internal/component/common/loki"
 	"github.com/grafana/alloy/internal/component/common/loki/positions"
 	"github.com/grafana/alloy/internal/component/common/loki/utils"
+	"github.com/grafana/alloy/internal/component/loki/source/file/internal/tail"
+	"github.com/grafana/alloy/internal/component/loki/source/file/internal/tail/watch"
 	"github.com/grafana/alloy/internal/runtime/logging/level"
 )
 
@@ -35,13 +34,13 @@ type tailer struct {
 	receiver  loki.LogsReceiver
 	positions positions.Positions
 
-	path               string
-	labelsStr          string
+	key                positions.Entry
 	labels             model.LabelSet
 	legacyPositionUsed bool
 
-	tailFromEnd bool
-	pollOptions watch.PollingFileWatcherOptions
+	tailFromEnd          bool
+	onPositionsFileError OnPositionsFileError
+	pollOptions          watch.PollingFileWatcherOptions
 
 	posAndSizeMtx sync.Mutex
 
@@ -49,38 +48,44 @@ type tailer struct {
 
 	componentStopping func() bool
 
+	report sync.Once
+
 	tail    *tail.Tail
 	decoder *encoding.Decoder
 }
 
 func newTailer(
-	metrics *metrics, logger log.Logger, receiver loki.LogsReceiver, positions positions.Positions, path string, labels model.LabelSet,
-	encoding string, pollOptions watch.PollingFileWatcherOptions, tailFromEnd bool, legacyPositonUsed bool, componentStopping func() bool,
+	metrics *metrics,
+	logger log.Logger,
+	receiver loki.LogsReceiver,
+	pos positions.Positions,
+	componentStopping func() bool,
+	opts sourceOptions,
 ) (*tailer, error) {
 
-	tailer := &tailer{
-		metrics:            metrics,
-		logger:             log.With(logger, "component", "tailer"),
-		receiver:           receiver,
-		positions:          positions,
-		path:               path,
-		labels:             labels,
-		labelsStr:          labels.String(),
-		running:            atomic.NewBool(false),
-		tailFromEnd:        tailFromEnd,
-		legacyPositionUsed: legacyPositonUsed,
-		pollOptions:        pollOptions,
-		componentStopping:  componentStopping,
+	decoder, err := getDecoder(opts.encoding)
+	if err != nil {
+		return nil, fmt.Errorf("failed to get decoder: %w", err)
 	}
 
-	if encoding != "" {
-		level.Info(tailer.logger).Log("msg", "Will decode messages", "from", encoding, "to", "UTF8")
-		encoder, err := ianaindex.IANA.Encoding(encoding)
-		if err != nil {
-			return nil, fmt.Errorf("failed to get IANA encoding %s: %w", encoding, err)
-		}
-		decoder := encoder.NewDecoder()
-		tailer.decoder = decoder
+	tailer := &tailer{
+		metrics:              metrics,
+		logger:               log.With(logger, "component", "tailer"),
+		receiver:             receiver,
+		positions:            pos,
+		key:                  positions.Entry{Path: opts.path, Labels: opts.labels.String()},
+		labels:               opts.labels,
+		running:              atomic.NewBool(false),
+		tailFromEnd:          opts.tailFromEnd,
+		legacyPositionUsed:   opts.legacyPositionUsed,
+		onPositionsFileError: opts.onPositionsFileError,
+		pollOptions: watch.PollingFileWatcherOptions{
+			MinPollFrequency: opts.fileWatch.MinPollFrequency,
+			MaxPollFrequency: opts.fileWatch.MaxPollFrequency,
+		},
+		componentStopping: componentStopping,
+		report:            sync.Once{},
+		decoder:           decoder,
 	}
 
 	return tailer, nil
@@ -151,12 +156,21 @@ func (t *tailer) Run(ctx context.Context) {
 	}
 
 	handler, err := t.initRun()
+
 	if err != nil {
-		level.Error(t.logger).Log("msg", "failed to run tailer", "err", err)
+		// We are retrying tailers until the target has disappeared.
+		// We are mostly interested in this log if this happens directly when
+		// the tailer is scheduled and not on retries.
+		t.report.Do(func() {
+			level.Error(t.logger).Log("msg", "failed to run tailer", "err", err)
+		})
 		return
 	}
 	defer handler.Stop()
 
+	// We call report so that retries won't log.
+	t.report.Do(func() {})
+
 	t.metrics.filesActive.Add(1.)
 
 	done := make(chan struct{})
@@ -175,20 +189,35 @@ func (t *tailer) Run(ctx context.Context) {
 }
 
 func (t *tailer) initRun() (loki.EntryHandler, error) {
-	fi, err := os.Stat(t.path)
+	fi, err := os.Stat(t.key.Path)
 	if err != nil {
 		return nil, fmt.Errorf("failed to tail file: %w", err)
 	}
 
-	pos, err := t.positions.Get(t.path, t.labelsStr)
+	pos, err := t.positions.Get(t.key.Path, t.key.Labels)
 	if err != nil {
-		return nil, fmt.Errorf("failed to get file position: %w", err)
+		switch t.onPositionsFileError {
+		case OnPositionsFileErrorSkip:
+			return nil, fmt.Errorf("failed to get file position: %w", err)
+		case OnPositionsFileErrorRestartEnd:
+			pos, err = getLastLinePosition(t.key.Path)
+			if err != nil {
+				return nil, fmt.Errorf("failed to get last line position after positions error: %w", err)
+			}
+			level.Info(t.logger).Log("msg", "retrieved the position of the last line after positions error")
+		default:
+			level.Debug(t.logger).Log("msg", "unrecognized `on_positions_file_error` option, defaulting to `restart_from_beginning`", "option", t.onPositionsFileError)
+			fallthrough
+		case OnPositionsFileErrorRestartBeginning:
+			pos = 0
+			level.Info(t.logger).Log("msg", "reset position to start of file after positions error")
+		}
 	}
 
 	// If we translated legacy positions we should try to get position offset without labels
 	// when no other position was matched.
 	if pos == 0 && t.legacyPositionUsed {
-		pos, err = t.positions.Get(t.path, "{}")
+		pos, err = t.positions.Get(t.key.Path, "{}")
 		if err != nil {
 			return nil, fmt.Errorf("failed to get file position with empty labels: %w", err)
 		}
@@ -200,31 +229,25 @@ func (t *tailer) initRun() (loki.EntryHandler, error) {
 	// mostly show up on Windows because on Unix systems, the readlines function is not exited on file rotation.
 	// If this ever becomes a problem, we may want to consider saving and comparing file creation timestamps.
 	if fi.Size() < pos {
-		t.positions.Remove(t.path, t.labelsStr)
+		t.positions.Remove(t.key.Path, t.key.Labels)
 	}
 
 	// If no cached position is found and the tailFromEnd option is enabled.
 	if pos == 0 && t.tailFromEnd {
-		pos, err = getLastLinePosition(t.path)
+		pos, err = getLastLinePosition(t.key.Path)
 		if err != nil {
 			level.Error(t.logger).Log("msg", "failed to get a position from the end of the file, default to start of file", err)
 		} else {
-			t.positions.Put(t.path, t.labelsStr, pos)
+			t.positions.Put(t.key.Path, t.key.Labels, pos)
 			level.Info(t.logger).Log("msg", "retrieved and stored the position of the last line")
 		}
 	}
 
-	tail, err := tail.TailFile(t.path, tail.Config{
-		Follow:    true,
-		Poll:      true,
-		ReOpen:    true,
-		MustExist: true,
-		Location: &tail.SeekInfo{
-			Offset: pos,
-			Whence: 0,
-		},
-		Logger:      util.NewLogAdapter(t.logger),
+	tail, err := tail.TailFile(t.key.Path, tail.Config{
+		Location:    &tail.SeekInfo{Offset: pos, Whence: 0},
+		Logger:      t.logger,
 		PollOptions: t.pollOptions,
+		Decoder:     t.decoder,
 	})
 
 	if err != nil {
@@ -233,12 +256,24 @@ func (t *tailer) initRun() (loki.EntryHandler, error) {
 
 	t.tail = tail
 
-	labelsMiddleware := t.labels.Merge(model.LabelSet{filenameLabel: model.LabelValue(t.path)})
+	labelsMiddleware := t.labels.Merge(model.LabelSet{labelFilename: model.LabelValue(t.key.Path)})
 	handler := loki.AddLabelsMiddleware(labelsMiddleware).Wrap(loki.NewEntryHandler(t.receiver.Chan(), func() {}))
 
 	return handler, nil
 }
 
+func getDecoder(encoding string) (*encoding.Decoder, error) {
+	if encoding == "" {
+		return nil, nil
+	}
+
+	encoder, err := ianaindex.IANA.Encoding(encoding)
+	if err != nil {
+		return nil, fmt.Errorf("failed to get IANA encoding %s: %w", encoding, err)
+	}
+	return encoder.NewDecoder(), nil
+}
+
 // updatePosition is run in a goroutine and checks the current size of the file
 // and saves it to the positions file at a regular interval. If there is ever
 // an error it stops the tailer and exits, the tailer will be re-opened by the
@@ -249,7 +284,7 @@ func (t *tailer) updatePosition(posquit chan struct{}) {
 	positionWait := time.NewTicker(positionSyncPeriod)
 	defer func() {
 		positionWait.Stop()
-		level.Info(t.logger).Log("msg", "position timer: exited", "path", t.path)
+		level.Info(t.logger).Log("msg", "position timer: exited", "path", t.key.Path)
 		// NOTE: metrics must be cleaned up after the position timer exits, as markPositionAndSize() updates metrics.
 		t.cleanupMetrics()
 	}()
@@ -259,10 +294,10 @@ func (t *tailer) updatePosition(posquit chan struct{}) {
 		case <-positionWait.C:
 			err := t.markPositionAndSize()
 			if err != nil {
-				level.Error(t.logger).Log("msg", "position timer: error getting tail position and/or size, stopping tailer", "path", t.path, "error", err)
+				level.Error(t.logger).Log("msg", "position timer: error getting tail position and/or size, stopping tailer", "path", t.key.Path, "error", err)
 				err := t.tail.Stop()
 				if err != nil {
-					level.Error(t.logger).Log("msg", "position timer: error stopping tailer", "path", t.path, "error", err)
+					level.Error(t.logger).Log("msg", "position timer: error stopping tailer", "path", t.key.Path, "error", err)
 				}
 				return
 			}
@@ -279,7 +314,7 @@ func (t *tailer) updatePosition(posquit chan struct{}) {
 // called, the underlying tailer will never exit if there are unread lines in
 // the t.tail.Lines channel
 func (t *tailer) readLines(handler loki.EntryHandler, done chan struct{}) {
-	level.Info(t.logger).Log("msg", "tail routine: started", "path", t.path)
+	level.Info(t.logger).Log("msg", "tail routine: started", "path", t.key.Path)
 
 	posquit, posdone := make(chan struct{}), make(chan struct{})
 	go func() {
@@ -290,7 +325,7 @@ func (t *tailer) readLines(handler loki.EntryHandler, done chan struct{}) {
 	// This function runs in a goroutine, if it exits this tailer will never do any more tailing.
 	// Clean everything up.
 	defer func() {
-		level.Info(t.logger).Log("msg", "tail routine: exited", "path", t.path)
+		level.Info(t.logger).Log("msg", "tail routine: exited", "path", t.key.Path)
 		// Shut down the position marker thread
 		close(posquit)
 		<-posdone
@@ -301,37 +336,18 @@ func (t *tailer) readLines(handler loki.EntryHandler, done chan struct{}) {
 	for {
 		line, ok := <-t.tail.Lines
 		if !ok {
-			level.Info(t.logger).Log("msg", "tail routine: tail channel closed, stopping tailer", "path", t.path, "reason", t.tail.Tomb.Err())
+			level.Info(t.logger).Log("msg", "tail routine: tail channel closed, stopping tailer", "path", t.key.Path, "reason", t.tail.Tomb.Err())
 			return
 		}
 
-		// Note currently the tail implementation hardcodes Err to nil, this should never hit.
-		if line.Err != nil {
-			level.Error(t.logger).Log("msg", "tail routine: error reading line", "path", t.path, "error", line.Err)
-			continue
-		}
-
-		var text string
-		if t.decoder != nil {
-			var err error
-			text, err = t.convertToUTF8(line.Text)
-			if err != nil {
-				level.Debug(t.logger).Log("msg", "failed to convert encoding", "error", err)
-				t.metrics.encodingFailures.WithLabelValues(t.path).Inc()
-				text = fmt.Sprintf("the requested encoding conversion for this line failed in Alloy: %s", err.Error())
-			}
-		} else {
-			text = line.Text
-		}
-
-		t.metrics.readLines.WithLabelValues(t.path).Inc()
+		t.metrics.readLines.WithLabelValues(t.key.Path).Inc()
 		entries <- loki.Entry{
 			// Allocate the expected size of labels. This matches the number of labels added by the middleware
 			// as configured in initRun().
 			Labels: make(model.LabelSet, len(t.labels)+1),
-			Entry: logproto.Entry{
+			Entry: push.Entry{
 				Timestamp: line.Time,
-				Line:      text,
+				Line:      line.Text,
 			},
 		}
 	}
@@ -346,7 +362,7 @@ func (t *tailer) markPositionAndSize() error {
 	if err != nil {
 		// If the file no longer exists, no need to save position information
 		if err == os.ErrNotExist {
-			level.Info(t.logger).Log("msg", "skipping update of position for a file which does not currently exist", "path", t.path)
+			level.Info(t.logger).Log("msg", "skipping update of position for a file which does not currently exist", "path", t.key.Path)
 			return nil
 		}
 		return err
@@ -358,9 +374,9 @@ func (t *tailer) markPositionAndSize() error {
 	}
 
 	// Update metrics and positions file all together to avoid race conditions when `t.tail` is stopped.
-	t.metrics.totalBytes.WithLabelValues(t.path).Set(float64(size))
-	t.metrics.readBytes.WithLabelValues(t.path).Set(float64(pos))
-	t.positions.Put(t.path, t.labelsStr, pos)
+	t.metrics.totalBytes.WithLabelValues(t.key.Path).Set(float64(size))
+	t.metrics.readBytes.WithLabelValues(t.key.Path).Set(float64(pos))
+	t.positions.Put(t.key.Path, t.key.Labels, pos)
 
 	return nil
 }
@@ -368,55 +384,47 @@ func (t *tailer) markPositionAndSize() error {
 func (t *tailer) stop(done chan struct{}) {
 	// Save the current position before shutting down tailer to ensure that if the file is tailed again
 	// it start where it left off.
-	err := t.markPositionAndSize()
-	if err != nil {
-		level.Error(t.logger).Log("msg", "error marking file position when stopping tailer", "path", t.path, "error", err)
+	if err := t.markPositionAndSize(); err != nil {
+		level.Error(t.logger).Log("msg", "error marking file position when stopping tailer", "path", t.key.Path, "error", err)
 	}
-
-	err = t.tail.Stop()
-	if err != nil {
+	if err := t.tail.Stop(); err != nil {
 		if utils.IsEphemeralOrFileClosed(err) {
 			// Don't log as error if the file is already closed, or we got an ephemeral error - it's a common case
 			// when files are rotating while being read and the tailer would have stopped correctly anyway.
-			level.Debug(t.logger).Log("msg", "tailer stopped with file I/O error", "path", t.path, "error", err)
-		} else {
+			level.Debug(t.logger).Log("msg", "tailer stopped with file I/O error", "path", t.key.Path, "error", err)
+		} else if !errors.Is(err, os.ErrNotExist) {
 			// Log as error for other reasons, as a resource leak may have happened.
-			level.Error(t.logger).Log("msg", "error stopping tailer", "path", t.path, "error", err)
+			level.Error(t.logger).Log("msg", "error stopping tailer", "path", t.key.Path, "error", err)
 		}
 	}
 
-	level.Debug(t.logger).Log("msg", "waiting for readline and position marker to exit", "path", t.path)
+	level.Debug(t.logger).Log("msg", "waiting for readline and position marker to exit", "path", t.key.Path)
 
 	// Wait for readLines() to consume all the remaining messages and exit when the channel is closed
 	<-done
 
-	level.Info(t.logger).Log("msg", "stopped tailing file", "path", t.path)
+	level.Info(t.logger).Log("msg", "stopped tailing file", "path", t.key.Path)
 
 	// If the component is not stopping, then it means that the target for this component is gone and that
 	// we should clear the entry from the positions file.
 	if !t.componentStopping() {
-		t.positions.Remove(t.path, t.labelsStr)
+		t.positions.Remove(t.key.Path, t.key.Labels)
 	}
 }
 
+func (t *tailer) Key() positions.Entry {
+	return t.key
+}
+
 func (t *tailer) IsRunning() bool {
 	return t.running.Load()
 }
 
-func (t *tailer) convertToUTF8(text string) (string, error) {
-	res, _, err := transform.String(t.decoder, text)
-	if err != nil {
-		return "", fmt.Errorf("failed to decode text to UTF8: %w", err)
-	}
-
-	return res, nil
-}
-
 // cleanupMetrics removes all metrics exported by this tailer
 func (t *tailer) cleanupMetrics() {
 	// When we stop tailing the file, also un-export metrics related to the file
 	t.metrics.filesActive.Add(-1.)
-	t.metrics.readLines.DeleteLabelValues(t.path)
-	t.metrics.readBytes.DeleteLabelValues(t.path)
-	t.metrics.totalBytes.DeleteLabelValues(t.path)
+	t.metrics.readLines.DeleteLabelValues(t.key.Path)
+	t.metrics.readBytes.DeleteLabelValues(t.key.Path)
+	t.metrics.totalBytes.DeleteLabelValues(t.key.Path)
 }
