@@ -18,11 +18,12 @@ func init() {
 	})
 }
 
-func createExporter(opts component.Options, args component.Arguments, defaultInstanceKey string) (integrations.Integration, string, error) {
+func createExporter(opts component.Options, args component.Arguments) (integrations.Integration, string, error) {
 	a := args.(Arguments)
 	cfg, err := a.Convert()
 	if err != nil {
 		return nil, "", err
 	}
+	defaultInstanceKey := opts.ID // use component ID if there is no better option
 	return integrations.NewIntegrationWithInstanceKey(opts.Logger, cfg, defaultInstanceKey)
 }
