@@ -92,7 +92,8 @@ func New(o component.Options, args Arguments) (*Component, error) {
 			log.With(o.Logger, "subcomponent", "rw"),
 		),
 	)
-	remoteStore := remote.NewStorage(remoteLogger, o.Registerer, startTime, o.DataPath, remoteFlushDeadline, nil)
+	// TODO: Expose the option to enable type and unit labels: https://github.com/grafana/alloy/issues/4659
+	remoteStore := remote.NewStorage(remoteLogger, o.Registerer, startTime, o.DataPath, remoteFlushDeadline, nil, false)
 
 	walStorage.SetNotifier(remoteStore)
 
@@ -125,10 +126,35 @@ func New(o component.Options, args Arguments) (*Component, error) {
 		debugDataPublisher: debugDataPublisher.(livedebugging.DebugDataPublisher),
 	}
 	componentID := livedebugging.ComponentID(res.opts.ID)
+
+	handleLocalLink := func(globalRef uint64, l labels.Labels, cachedLocalRef uint64, newLocalRef uint64) {
+		// We had a local ref that was still valid nothing to do
+		if cachedLocalRef != 0 && cachedLocalRef == newLocalRef {
+			return
+		}
+
+		// There are some unique scenarios that can have an append end with no error but the returned localRef is zero (duplicate exemplars).
+		// We don't want to update a valid link to an invalid link
+		if cachedLocalRef != 0 && newLocalRef == 0 {
+			return
+		}
+
+		// This should never happen in a proper appender chain. Since we cannot enforce it, we are extra defensive.
+		if globalRef == 0 {
+			globalRef = ls.GetOrAddGlobalRefID(l)
+		}
+
+		if cachedLocalRef == 0 {
+			ls.AddLocalLink(res.opts.ID, globalRef, newLocalRef)
+		} else {
+			ls.ReplaceLocalLink(res.opts.ID, globalRef, cachedLocalRef, newLocalRef)
+		}
+	}
+
 	res.receiver = prometheus.NewInterceptor(
 		res.storage,
 		ls,
-
+		prometheus.WithComponentID(res.opts.ID),
 		// In the methods below, conversion is needed because remote_writes assume
 		// they are responsible for generating ref IDs. This means two
 		// remote_writes may return the same ref ID for two different series. We
@@ -140,11 +166,12 @@ func New(o component.Options, args Arguments) (*Component, error) {
 				return 0, fmt.Errorf("%s has exited", o.ID)
 			}
 
-			localID := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
-			newRef, nextErr := next.Append(storage.SeriesRef(localID), l, t, v)
-			if localID == 0 {
-				ls.GetOrAddLink(res.opts.ID, uint64(newRef), l)
+			localRef := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
+			newLocalRef, nextErr := next.Append(storage.SeriesRef(localRef), l, t, v)
+			if nextErr == nil {
+				handleLocalLink(uint64(globalRef), l, localRef, uint64(newLocalRef))
 			}
+
 			res.debugDataPublisher.PublishIfActive(livedebugging.NewData(
 				componentID,
 				livedebugging.PrometheusMetric,
@@ -160,11 +187,12 @@ func New(o component.Options, args Arguments) (*Component, error) {
 				return 0, fmt.Errorf("%s has exited", o.ID)
 			}
 
-			localID := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
-			newRef, nextErr := next.AppendHistogram(storage.SeriesRef(localID), l, t, h, fh)
-			if localID == 0 {
-				ls.GetOrAddLink(res.opts.ID, uint64(newRef), l)
+			localRef := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
+			newLocalRef, nextErr := next.AppendHistogram(storage.SeriesRef(localRef), l, t, h, fh)
+			if nextErr == nil {
+				handleLocalLink(uint64(globalRef), l, localRef, uint64(newLocalRef))
 			}
+
 			res.debugDataPublisher.PublishIfActive(livedebugging.NewData(
 				componentID,
 				livedebugging.PrometheusMetric,
@@ -188,11 +216,12 @@ func New(o component.Options, args Arguments) (*Component, error) {
 				return 0, fmt.Errorf("%s has exited", o.ID)
 			}
 
-			localID := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
-			newRef, nextErr := next.UpdateMetadata(storage.SeriesRef(localID), l, m)
-			if localID == 0 {
-				ls.GetOrAddLink(res.opts.ID, uint64(newRef), l)
+			localRef := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
+			newLocalRef, nextErr := next.UpdateMetadata(storage.SeriesRef(localRef), l, m)
+			if nextErr == nil {
+				handleLocalLink(uint64(globalRef), l, localRef, uint64(newLocalRef))
 			}
+
 			res.debugDataPublisher.PublishIfActive(livedebugging.NewData(
 				componentID,
 				livedebugging.PrometheusMetric,
@@ -208,11 +237,12 @@ func New(o component.Options, args Arguments) (*Component, error) {
 				return 0, fmt.Errorf("%s has exited", o.ID)
 			}
 
-			localID := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
-			newRef, nextErr := next.AppendExemplar(storage.SeriesRef(localID), l, e)
-			if localID == 0 {
-				ls.GetOrAddLink(res.opts.ID, uint64(newRef), l)
+			localRef := ls.GetLocalRefID(res.opts.ID, uint64(globalRef))
+			newLocalRef, nextErr := next.AppendExemplar(storage.SeriesRef(localRef), l, e)
+			if nextErr == nil {
+				handleLocalLink(uint64(globalRef), l, localRef, uint64(newLocalRef))
 			}
+
 			res.debugDataPublisher.PublishIfActive(livedebugging.NewData(
 				componentID,
 				livedebugging.PrometheusMetric,
