@@ -16,6 +16,7 @@ import (
 
 	"github.com/goccy/go-json"
 	"go.opentelemetry.io/collector/pdata/pcommon"
+	"golang.org/x/exp/constraints"
 
 	"github.com/open-telemetry/opentelemetry-collector-contrib/pkg/ottl/internal/ottlcommon"
 )
@@ -52,6 +53,15 @@ type GetSetter[K any] interface {
 	Setter[K]
 }
 
+// literalGetter is an interface that allows Getter implementations to indicate if they
+// support literal values.
+type literalGetter interface {
+	// isLiteral returns true if the Getter is a literal value.
+	isLiteral() bool
+	// getLiteral retrieves the literal value of the Getter.
+	getLiteral() (any, error)
+}
+
 // StandardGetSetter is a standard way to construct a GetSetter
 type StandardGetSetter[K any] struct {
 	Getter func(ctx context.Context, tCtx K) (any, error)
@@ -74,6 +84,14 @@ func (l literal[K]) Get(context.Context, K) (any, error) {
 	return l.value, nil
 }
 
+func (l literal[K]) getLiteral() (any, error) {
+	return l.value, nil
+}
+
+func (literal[K]) isLiteral() bool {
+	return true
+}
+
 type exprGetter[K any] struct {
 	expr Expr[K]
 	keys []key
@@ -181,6 +199,22 @@ func (l *listGetter[K]) Get(ctx context.Context, tCtx K) (any, error) {
 	return evaluated, nil
 }
 
+func (l *listGetter[K]) isLiteral() bool {
+	if len(l.slice) == 0 {
+		return false
+	}
+	for _, v := range l.slice {
+		if getter, ok := v.(literalGetter); !ok || !getter.isLiteral() {
+			return false
+		}
+	}
+	return true
+}
+
+func (l *listGetter[K]) getLiteral() (any, error) {
+	return l.Get(context.Background(), *new(K))
+}
+
 type mapGetter[K any] struct {
 	mapValues map[string]Getter[K]
 }
@@ -220,6 +254,126 @@ func (m *mapGetter[K]) Get(ctx context.Context, tCtx K) (any, error) {
 	return result, nil
 }
 
+func (m *mapGetter[K]) isLiteral() bool {
+	if len(m.mapValues) == 0 {
+		return false
+	}
+	for _, v := range m.mapValues {
+		if getter, ok := v.(literalGetter); !ok || !getter.isLiteral() {
+			return false
+		}
+	}
+	return true
+}
+
+func (m *mapGetter[K]) getLiteral() (any, error) {
+	return m.Get(context.Background(), *new(K))
+}
+
+// PSliceGetter is a Getter that must return a pcommon.Slice.
+type PSliceGetter[K any] interface {
+	Get(ctx context.Context, tCtx K) (pcommon.Slice, error)
+}
+
+// newStandardPSliceGetter creates a new StandardPSliceGetter from a Getter[K],
+// also checking if the Getter is a literalGetter.
+func newStandardPSliceGetter[K any](getter Getter[K]) StandardPSliceGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardPSliceGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
+// StandardPSliceGetter is a basic implementation of PSliceGetter
+type StandardPSliceGetter[K any] struct {
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
+}
+
+// Get retrieves a pcommon.Slice value.
+// If the value is not a pcommon.Slice a new TypeError is returned.
+// If there is an error getting the value it will be returned.
+func (g StandardPSliceGetter[K]) Get(ctx context.Context, tCtx K) (pcommon.Slice, error) {
+	val, err := g.Getter(ctx, tCtx)
+	if err != nil {
+		return pcommon.Slice{}, fmt.Errorf("error getting value in %T: %w", g, err)
+	}
+	if val == nil {
+		return pcommon.Slice{}, TypeError("expected pcommon.Slice but got nil")
+	}
+	switch v := val.(type) {
+	case pcommon.Slice:
+		return v, nil
+	case pcommon.Value:
+		if v.Type() == pcommon.ValueTypeSlice {
+			return v.Slice(), nil
+		}
+		return pcommon.Slice{}, TypeError(fmt.Sprintf("expected pcommon.Slice but got %v", v.Type()))
+	case []any:
+		s := pcommon.NewSlice()
+		err = s.FromRaw(v)
+		if err != nil {
+			return pcommon.Slice{}, err
+		}
+		return s, nil
+	// Handle common slice types returned by OTTL functions
+	case []string:
+		return newPSliceFrom(v, func(target *pcommon.Value, value string) { target.SetStr(value) })
+	case []int:
+		return newPSliceFromIntegers(v)
+	case []int16:
+		return newPSliceFromIntegers(v)
+	case []int32:
+		return newPSliceFromIntegers(v)
+	case []int64:
+		return newPSliceFromIntegers(v)
+	case []uint:
+		return newPSliceFromIntegers(v)
+	case []uint16:
+		return newPSliceFromIntegers(v)
+	case []uint32:
+		return newPSliceFromIntegers(v)
+	case []uint64:
+		return newPSliceFromIntegers(v)
+	case []float32:
+		return newPSliceFrom(v, func(target *pcommon.Value, value float32) { target.SetDouble(float64(value)) })
+	case []float64:
+		return newPSliceFrom(v, func(target *pcommon.Value, value float64) { target.SetDouble(value) })
+	case []bool:
+		return newPSliceFrom(v, func(target *pcommon.Value, value bool) { target.SetBool(value) })
+	default:
+		return pcommon.Slice{}, TypeError(fmt.Sprintf("expected pcommon.Slice but got %T", val))
+	}
+}
+
+func (g StandardPSliceGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
+func (g StandardPSliceGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardPSliceGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func newPSliceFromIntegers[T constraints.Integer](source []T) (pcommon.Slice, error) {
+	return newPSliceFrom(source, func(target *pcommon.Value, value T) {
+		target.SetInt(int64(value))
+	})
+}
+
+func newPSliceFrom[T any](source []T, set func(target *pcommon.Value, value T)) (pcommon.Slice, error) {
+	s := pcommon.NewSlice()
+	s.EnsureCapacity(len(source))
+	for _, v := range source {
+		empty := s.AppendEmpty()
+		set(&empty, v)
+	}
+	return s, nil
+}
+
 // TypeError represents that a value was not an expected type.
 type TypeError string
 
@@ -233,9 +387,20 @@ type StringGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) (string, error)
 }
 
+// newStandardStringGetter creates a new StandardStringGetter from a Getter[K],
+// also checking if the Getter is a literalGetter.
+func newStandardStringGetter[K any](getter Getter[K]) StandardStringGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardStringGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardStringGetter is a basic implementation of StringGetter
 type StandardStringGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 // Get retrieves a string value.
@@ -262,15 +427,37 @@ func (g StandardStringGetter[K]) Get(ctx context.Context, tCtx K) (string, error
 	}
 }
 
+func (g StandardStringGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
+func (g StandardStringGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardStringGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
 // IntGetter is a Getter that must return an int64.
 type IntGetter[K any] interface {
 	// Get retrieves an int64 value.
 	Get(ctx context.Context, tCtx K) (int64, error)
 }
 
+// newStandardIntGetter creates a new StandardIntGetter from a Getter[K],
+// also checking if the Getter is a literalGetter.
+func newStandardIntGetter[K any](getter Getter[K]) StandardIntGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardIntGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardIntGetter is a basic implementation of IntGetter
 type StandardIntGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 // Get retrieves an int64 value.
@@ -297,15 +484,35 @@ func (g StandardIntGetter[K]) Get(ctx context.Context, tCtx K) (int64, error) {
 	}
 }
 
+func (g StandardIntGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
+func (g StandardIntGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardIntGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
 // FloatGetter is a Getter that must return a float64.
 type FloatGetter[K any] interface {
 	// Get retrieves a float64 value.
 	Get(ctx context.Context, tCtx K) (float64, error)
 }
 
+func newStandardFloatGetter[K any](getter Getter[K]) StandardFloatGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardFloatGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardFloatGetter is a basic implementation of FloatGetter
 type StandardFloatGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 // Get retrieves a float64 value.
@@ -332,15 +539,35 @@ func (g StandardFloatGetter[K]) Get(ctx context.Context, tCtx K) (float64, error
 	}
 }
 
+func (g StandardFloatGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
+func (g StandardFloatGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardFloatGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
 // BoolGetter is a Getter that must return a bool.
 type BoolGetter[K any] interface {
 	// Get retrieves a bool value.
 	Get(ctx context.Context, tCtx K) (bool, error)
 }
 
+func newStandardBoolGetter[K any](getter Getter[K]) StandardBoolGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardBoolGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardBoolGetter is a basic implementation of BoolGetter
 type StandardBoolGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 // Get retrieves a bool value.
@@ -367,6 +594,17 @@ func (g StandardBoolGetter[K]) Get(ctx context.Context, tCtx K) (bool, error) {
 	}
 }
 
+func (g StandardBoolGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
+func (g StandardBoolGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardBoolGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
 // FunctionGetter uses a function factory to return an instantiated function as an Expr.
 type FunctionGetter[K any] interface {
 	// Get returns a function as an Expr[K] built with the provided Arguments
@@ -436,9 +674,20 @@ type PMapGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) (pcommon.Map, error)
 }
 
+// newStandardPMapGetter creates a new StandardPMapGetter from a Getter[K],
+// also checking if the Getter is a literalGetter.
+func newStandardPMapGetter[K any](getter Getter[K]) StandardPMapGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardPMapGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardPMapGetter is a basic implementation of PMapGetter
 type StandardPMapGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 // Get retrieves a pcommon.Map value.
@@ -472,6 +721,17 @@ func (g StandardPMapGetter[K]) Get(ctx context.Context, tCtx K) (pcommon.Map, er
 	}
 }
 
+func (g StandardPMapGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardPMapGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardPMapGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
 // StringLikeGetter is a Getter that returns a string by converting the underlying value to a string if necessary.
 type StringLikeGetter[K any] interface {
 	// Get retrieves a string value.
@@ -481,9 +741,20 @@ type StringLikeGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) (*string, error)
 }
 
+// newStandardStringLikeGetter creates a new StandardStringLikeGetter from a Getter[K],
+// also checking if the Getter is a literalGetter.
+func newStandardStringLikeGetter[K any](getter Getter[K]) StandardStringLikeGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardStringLikeGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardStringLikeGetter is a basic implementation of StringLikeGetter
 type StandardStringLikeGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 func (g StandardStringLikeGetter[K]) Get(ctx context.Context, tCtx K) (*string, error) {
@@ -524,6 +795,17 @@ func (g StandardStringLikeGetter[K]) Get(ctx context.Context, tCtx K) (*string,
 	return &result, nil
 }
 
+func (g StandardStringLikeGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardStringLikeGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardStringLikeGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
 // FloatLikeGetter is a Getter that returns a float64 by converting the underlying value to a float64 if necessary.
 type FloatLikeGetter[K any] interface {
 	// Get retrieves a float64 value.
@@ -533,9 +815,18 @@ type FloatLikeGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) (*float64, error)
 }
 
+func newStandardFloatLikeGetter[K any](getter Getter[K]) StandardFloatLikeGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardFloatLikeGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardFloatLikeGetter is a basic implementation of FloatLikeGetter
 type StandardFloatLikeGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 func (g StandardFloatLikeGetter[K]) Get(ctx context.Context, tCtx K) (*float64, error) {
@@ -589,6 +880,17 @@ func (g StandardFloatLikeGetter[K]) Get(ctx context.Context, tCtx K) (*float64,
 	return &result, nil
 }
 
+func (g StandardFloatLikeGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardFloatLikeGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardFloatLikeGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
 // IntLikeGetter is a Getter that returns an int by converting the underlying value to an int if necessary
 type IntLikeGetter[K any] interface {
 	// Get retrieves an int value.
@@ -598,9 +900,18 @@ type IntLikeGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) (*int64, error)
 }
 
+func newStandardIntLikeGetter[K any](getter Getter[K]) StandardIntLikeGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardIntLikeGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardIntLikeGetter is a basic implementation of IntLikeGetter
 type StandardIntLikeGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 func (g StandardIntLikeGetter[K]) Get(ctx context.Context, tCtx K) (*int64, error) {
@@ -654,6 +965,17 @@ func (g StandardIntLikeGetter[K]) Get(ctx context.Context, tCtx K) (*int64, erro
 	return &result, nil
 }
 
+func (g StandardIntLikeGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardIntLikeGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardIntLikeGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
 // ByteSliceLikeGetter is a Getter that returns []byte by converting the underlying value to an []byte if necessary
 type ByteSliceLikeGetter[K any] interface {
 	// Get retrieves []byte value.
@@ -663,9 +985,18 @@ type ByteSliceLikeGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) ([]byte, error)
 }
 
+func newStandardByteSliceLikeGetter[K any](getter Getter[K]) StandardByteSliceLikeGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardByteSliceLikeGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardByteSliceLikeGetter is a basic implementation of ByteSliceLikeGetter
 type StandardByteSliceLikeGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 func (g StandardByteSliceLikeGetter[K]) Get(ctx context.Context, tCtx K) ([]byte, error) {
@@ -717,6 +1048,17 @@ func (g StandardByteSliceLikeGetter[K]) Get(ctx context.Context, tCtx K) ([]byte
 	return result, nil
 }
 
+func (g StandardByteSliceLikeGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardByteSliceLikeGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardByteSliceLikeGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
 // valueToBytes converts a value to a byte slice of length 8.
 func valueToBytes(n any) ([]byte, error) {
 	// Create a buffer to hold the bytes
@@ -739,9 +1081,18 @@ type BoolLikeGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) (*bool, error)
 }
 
+func newStandardBoolLikeGetter[K any](getter Getter[K]) StandardBoolLikeGetter[K] {
+	litGetter, isLiteralGetter := getter.(literalGetter)
+	return StandardBoolLikeGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteralGetter && litGetter.isLiteral(),
+	}
+}
+
 // StandardBoolLikeGetter is a basic implementation of BoolLikeGetter
 type StandardBoolLikeGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 func (g StandardBoolLikeGetter[K]) Get(ctx context.Context, tCtx K) (*bool, error) {
@@ -789,6 +1140,17 @@ func (g StandardBoolLikeGetter[K]) Get(ctx context.Context, tCtx K) (*bool, erro
 	return &result, nil
 }
 
+func (g StandardBoolLikeGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardBoolLikeGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardBoolLikeGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
 func (p *Parser[K]) newGetter(val value) (Getter[K], error) {
 	if val.IsNil != nil && *val.IsNil {
 		return &literal[K]{value: nil}, nil
@@ -801,7 +1163,7 @@ func (p *Parser[K]) newGetter(val value) (Getter[K], error) {
 		return &literal[K]{value: bool(*b)}, nil
 	}
 	if b := val.Bytes; b != nil {
-		return &literal[K]{value: ([]byte)(*b)}, nil
+		return &literal[K]{value: []byte(*b)}, nil
 	}
 
 	if val.Enum != nil {
@@ -879,9 +1241,18 @@ type TimeGetter[K any] interface {
 	Get(ctx context.Context, tCtx K) (time.Time, error)
 }
 
+func newStandardTimeGetter[K any](getter Getter[K]) StandardTimeGetter[K] {
+	litGetter, isLiteral := getter.(literalGetter)
+	return StandardTimeGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteral && litGetter.isLiteral(),
+	}
+}
+
 // StandardTimeGetter is a basic implementation of TimeGetter
 type StandardTimeGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 // Get retrieves a time.Time value.
@@ -903,15 +1274,35 @@ func (g StandardTimeGetter[K]) Get(ctx context.Context, tCtx K) (time.Time, erro
 	}
 }
 
+func (g StandardTimeGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardTimeGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardTimeGetter[K]) isLiteral() bool {
+	return g.literal
+}
+
 // DurationGetter is a Getter that must return a time.Duration.
 type DurationGetter[K any] interface {
 	// Get retrieves a time.Duration value.
 	Get(ctx context.Context, tCtx K) (time.Duration, error)
 }
 
+func newStandardDurationGetter[K any](getter Getter[K]) StandardDurationGetter[K] {
+	litGetter, isLiteral := getter.(literalGetter)
+	return StandardDurationGetter[K]{
+		Getter:  getter.Get,
+		literal: isLiteral && litGetter.isLiteral(),
+	}
+}
+
 // StandardDurationGetter is a basic implementation of DurationGetter
 type StandardDurationGetter[K any] struct {
-	Getter func(ctx context.Context, tCtx K) (any, error)
+	Getter  func(ctx context.Context, tCtx K) (any, error)
+	literal bool
 }
 
 // Get retrieves an time.Duration value.
@@ -932,3 +1323,14 @@ func (g StandardDurationGetter[K]) Get(ctx context.Context, tCtx K) (time.Durati
 		return 0, TypeError(fmt.Sprintf("expected duration but got %T", val))
 	}
 }
+
+func (g StandardDurationGetter[K]) getLiteral() (any, error) {
+	if !g.literal {
+		return nil, errors.New("StandardDurationGetter value is not a literal")
+	}
+	return g.Get(context.Background(), *new(K))
+}
+
+func (g StandardDurationGetter[K]) isLiteral() bool {
+	return g.literal
+}
