@@ -9,6 +9,8 @@ import (
 	"go.opentelemetry.io/collector/component"
 	"go.opentelemetry.io/collector/pdata/pcommon"
 	"go.uber.org/zap"
+
+	"github.com/open-telemetry/opentelemetry-collector-contrib/processor/tailsamplingprocessor/pkg/samplingpolicy"
 )
 
 type spanCount struct {
@@ -17,10 +19,10 @@ type spanCount struct {
 	maxSpans int32
 }
 
-var _ PolicyEvaluator = (*spanCount)(nil)
+var _ samplingpolicy.Evaluator = (*spanCount)(nil)
 
 // NewSpanCount creates a policy evaluator sampling traces with more than one span per trace
-func NewSpanCount(settings component.TelemetrySettings, minSpans, maxSpans int32) PolicyEvaluator {
+func NewSpanCount(settings component.TelemetrySettings, minSpans, maxSpans int32) samplingpolicy.Evaluator {
 	return &spanCount{
 		logger:   settings.Logger,
 		minSpans: minSpans,
@@ -29,16 +31,16 @@ func NewSpanCount(settings component.TelemetrySettings, minSpans, maxSpans int32
 }
 
 // Evaluate looks at the trace data and returns a corresponding SamplingDecision.
-func (c *spanCount) Evaluate(_ context.Context, _ pcommon.TraceID, traceData *TraceData) (Decision, error) {
+func (c *spanCount) Evaluate(_ context.Context, _ pcommon.TraceID, traceData *samplingpolicy.TraceData) (samplingpolicy.Decision, error) {
 	c.logger.Debug("Evaluating spans counts in filter")
 
 	spanCount := int(traceData.SpanCount.Load())
 	switch {
 	case c.maxSpans == 0 && spanCount >= int(c.minSpans):
-		return Sampled, nil
+		return samplingpolicy.Sampled, nil
 	case spanCount >= int(c.minSpans) && spanCount <= int(c.maxSpans):
-		return Sampled, nil
+		return samplingpolicy.Sampled, nil
 	default:
-		return NotSampled, nil
+		return samplingpolicy.NotSampled, nil
 	}
 }
