@@ -6,6 +6,8 @@ package sampling // import "github.com/open-telemetry/opentelemetry-collector-co
 import (
 	"go.opentelemetry.io/collector/pdata/pcommon"
 	"go.opentelemetry.io/collector/pdata/ptrace"
+
+	"github.com/open-telemetry/opentelemetry-collector-contrib/processor/tailsamplingprocessor/pkg/samplingpolicy"
 )
 
 // hasResourceOrSpanWithCondition iterates through all the resources and instrumentation library spans until any
@@ -14,20 +16,20 @@ func hasResourceOrSpanWithCondition(
 	td ptrace.Traces,
 	shouldSampleResource func(resource pcommon.Resource) bool,
 	shouldSampleSpan func(span ptrace.Span) bool,
-) Decision {
+) samplingpolicy.Decision {
 	for i := 0; i < td.ResourceSpans().Len(); i++ {
 		rs := td.ResourceSpans().At(i)
 
 		resource := rs.Resource()
 		if shouldSampleResource(resource) {
-			return Sampled
+			return samplingpolicy.Sampled
 		}
 
 		if hasInstrumentationLibrarySpanWithCondition(rs.ScopeSpans(), shouldSampleSpan, false) {
-			return Sampled
+			return samplingpolicy.Sampled
 		}
 	}
-	return NotSampled
+	return samplingpolicy.NotSampled
 }
 
 // invertHasResourceOrSpanWithCondition iterates through all the resources and instrumentation library spans until any
@@ -36,7 +38,7 @@ func invertHasResourceOrSpanWithCondition(
 	td ptrace.Traces,
 	shouldSampleResource func(resource pcommon.Resource) bool,
 	shouldSampleSpan func(span ptrace.Span) bool,
-) Decision {
+) samplingpolicy.Decision {
 	isd := IsInvertDecisionsDisabled()
 
 	for i := 0; i < td.ResourceSpans().Len(); i++ {
@@ -45,35 +47,35 @@ func invertHasResourceOrSpanWithCondition(
 		resource := rs.Resource()
 		if !shouldSampleResource(resource) {
 			if isd {
-				return NotSampled
+				return samplingpolicy.NotSampled
 			}
-			return InvertNotSampled
+			return samplingpolicy.InvertNotSampled
 		}
 
 		if !hasInstrumentationLibrarySpanWithCondition(rs.ScopeSpans(), shouldSampleSpan, true) {
 			if isd {
-				return NotSampled
+				return samplingpolicy.NotSampled
 			}
-			return InvertNotSampled
+			return samplingpolicy.InvertNotSampled
 		}
 	}
 
 	if isd {
-		return Sampled
+		return samplingpolicy.Sampled
 	}
-	return InvertSampled
+	return samplingpolicy.InvertSampled
 }
 
 // hasSpanWithCondition iterates through all the instrumentation library spans until any callback returns true.
-func hasSpanWithCondition(td ptrace.Traces, shouldSample func(span ptrace.Span) bool) Decision {
+func hasSpanWithCondition(td ptrace.Traces, shouldSample func(span ptrace.Span) bool) samplingpolicy.Decision {
 	for i := 0; i < td.ResourceSpans().Len(); i++ {
 		rs := td.ResourceSpans().At(i)
 
 		if hasInstrumentationLibrarySpanWithCondition(rs.ScopeSpans(), shouldSample, false) {
-			return Sampled
+			return samplingpolicy.Sampled
 		}
 	}
-	return NotSampled
+	return samplingpolicy.NotSampled
 }
 
 func hasInstrumentationLibrarySpanWithCondition(ilss ptrace.ScopeSpansSlice, check func(span ptrace.Span) bool, invert bool) bool {
@@ -91,7 +93,7 @@ func hasInstrumentationLibrarySpanWithCondition(ilss ptrace.ScopeSpansSlice, che
 	return invert
 }
 
-func SetAttrOnScopeSpans(data *TraceData, attrName, attrKey string) {
+func SetAttrOnScopeSpans(data *samplingpolicy.TraceData, attrName, attrKey string) {
 	data.Lock()
 	defer data.Unlock()
 
@@ -104,3 +106,14 @@ func SetAttrOnScopeSpans(data *TraceData, attrName, attrKey string) {
 		}
 	}
 }
+
+func SetBoolAttrOnScopeSpans(data ptrace.Traces, attrName string, attrValue bool) {
+	rs := data.ResourceSpans()
+	for i := 0; i < rs.Len(); i++ {
+		rss := rs.At(i)
+		for j := 0; j < rss.ScopeSpans().Len(); j++ {
+			ss := rss.ScopeSpans().At(j)
+			ss.Scope().Attributes().PutBool(attrName, attrValue)
+		}
+	}
+}
